

# encoding: utf-8
import c4d
from cioc4d.widgets import utils as wutil


class PathWidget(object):
    """A widget containing a label and textfield with a delete btn.
    
    Inteded to appear in a list, such as extra uploads.
    """

    def __init__(self, dialog, path):

        self.dialog = dialog
        self.group_id = None
        self.delete_btn_id = None
        self.text_field_id = None

        self._build()
        self.set_value(path)

    def _build(self):

        with wutil.grid_group(self.dialog, cols=2) as group_id:
            self.group_id = group_id
            self.delete_btn_id = self.dialog.register()
            self.dialog.AddButton(self.delete_btn_id,
                                  c4d.BFH_FIT, initw=0, name='X')

            self.text_field_id = self.dialog.register()
            self.dialog.AddEditText(
                self.text_field_id, c4d.BFH_SCALEFIT | c4d.BFV_TOP)

    def set_value(self, value):
        self.dialog.SetString(self.text_field_id, value)

    def get_value(self):
        return self.dialog.GetString(self.text_field_id).strip()



import os
import c4d
from cioc4d.sections.collapsible_section import CollapsibleSection
from cioc4d.widgets.button_row import ButtonRow
from cioc4d.widgets.path_widget import PathWidget
from ciocore.gpath_list import GLOBBABLE_REGEX, PathList
from cioc4d import const as k

BG_COLOR = c4d.Vector(0.2, 0.2, 0.2)


class AssetsSection(CollapsibleSection):
    ORDER = 200

    def __init__(self, dialog):
        self.path_list_grp_id = None
        self.pathlist = PathList()
        self.path_widgets = []
        super(AssetsSection, self).__init__(
            dialog, "Extra Assets", collapse=True)

    def build(self):

        self.button_row = ButtonRow(
            self.dialog, "Clear List", "Browse File", "Browse Directory", label="Actions")
            
        self.scroll_id = self.dialog.register()
        self.path_list_grp_id = self.dialog.register()

        scroll_grp = self.dialog.ScrollGroupBegin(
            id=self.scroll_id,
            flags=c4d.BFH_SCALEFIT,
            inith=100,
            scrollflags=c4d.SCROLLGROUP_VERT | c4d.SCROLLGROUP_HORIZ)

        if scroll_grp:
            grp = self.dialog.GroupBegin(
                id=self.path_list_grp_id,
                flags=c4d.BFH_SCALEFIT | c4d.BFV_SCALEFIT,
                title="",
                cols=1,
                groupflags=0)
            if grp:
                self.dialog.GroupBorderSpace(2, 2, 2, 2)

            self.dialog.GroupEnd()  # main
        self.dialog.GroupEnd()  # scroll

        self.dialog.SetDefaultColor(
            self.path_list_grp_id, c4d.COLOR_BG,   BG_COLOR)

    def populate_from_store(self):
        store = self.dialog.store
        self.pathlist = PathList(*(store.assets()))
        self._update_widgets()

    def save_to_store(self):
        store = self.dialog.store
        store.set_assets([p.posix_path() for p in self.pathlist])
        store.commit()

    @property
    def clear_button_id(self):
        return self.button_row.button_ids[0]

    @property
    def browse_file_button_id(self):
        return self.button_row.button_ids[1]

    @property
    def browse_directory_button_id(self):
        return self.button_row.button_ids[2]

    # @property
    # def preview_button_id(self):
    #     return self.button_row.button_ids[3]

    def on_plugin_message(self, widget_id, msg):

        # clear list button
        if widget_id == self.clear_button_id:
            self.clear_entries()
         # browse file
        elif widget_id == self.browse_file_button_id:
            fn = c4d.storage.LoadDialog(flags=c4d.FILESELECT_LOAD)
            self.add_entry(fn)
        # browse dir
        elif widget_id == self.browse_directory_button_id:
            fn = c4d.storage.LoadDialog(flags=c4d.FILESELECT_DIRECTORY)
            self.add_entry(fn)
        else:
            # delete one entry button
            path_widget = next(
                (widget for widget in self.path_widgets if widget.delete_btn_id == widget_id), None)
            if path_widget:
                self.remove_entry(path_widget.get_value())

    def clear_entries(self):
        self.pathlist = PathList()
        self._update_widgets()
        self.save_to_store()

    def add_entry(self, entry):
        self.pathlist.add(entry.decode("utf8"))
        self._update_widgets()
        self.save_to_store()

    def remove_entry(self, entry):
        self.pathlist.remove(entry.decode("utf8"))
        self._update_widgets()
        self.save_to_store()

    def _update_widgets(self):
        self.path_widgets = []
        self.dialog.LayoutFlushGroup(self.path_list_grp_id)
        for path in self.pathlist:
            self.path_widgets.append(
                PathWidget(self.dialog, path=path.posix_path()))
        self.dialog.LayoutChanged(self.path_list_grp_id)
        

    def resolve(self, _, **kwargs):

        if not kwargs.get("with_assets"):
            return {
                "upload_paths":  "Assets are only shown here if you use the Show Assets button above."}
        try:
            return {
                "upload_paths": self.get_upload_paths()
            }
        except ValueError as ex:
            pass

        return {
            "upload_paths":  str(ex)
        }
 

    def get_upload_paths(self):
        return sorted([p.posix_path() for p in self.get_assets_path_list()])


    def get_assets_path_list(self):
        # the extra assets
        # copy

        document = c4d.documents.GetActiveDocument()

        path_list= PathList(*self.pathlist) 

        # Called remove_missing here because its cheaper to do it while the list
        # is small. The scraped assets have an exists field anyway.
        path_list.remove_missing()

        if k.C4D_VERSION < 22:
            asset_list = c4d.documents.GetAllAssets(document, False, '')
            path_list.add(*[asset["filename"]  for asset in asset_list if asset["exists"]  ] )
        else:
            asset_list = []
            success = c4d.documents.GetAllAssetsNew(
                document, False, '',
                flags=c4d.ASSETDATA_FLAG_WITHCACHES,
                assetList=asset_list)
            if success == c4d.GETALLASSETSRESULT_FAILED:
                raise ValueError("c4d.GetAllAssetsNew gave an error.")
            else:
                path_list.add(*[asset["filename"]
                                for asset in asset_list if asset["exists"]])

        path_list.glob()
        return path_list
        


 

# def _remove_missing_files(pathlist):
#     # NOTE: This is temporary - SHOULD BE HANDLED IN GPathList class. We
#     # remove any non-globbable paths that dont exist. We never want the
#     # uploader to fail because of missing files. The reason is that the
#     # uploader output is not user friendly and its hard to find the
#     # culprits. Instead we alert the user about missing files and ask them
#     # if they want to continue.
#     result = PathList()
#     for path in pathlist:
#         pp = path.posix_path()
#         if GLOBBABLE_REGEX.search(pp):
#             result.add(path)
#         else:
#             if os.path.exists(pp):
#                 result.add(path)
#     return result





import c4d
from cioc4d.widgets import utils as wutil


class ComboBoxGrp(object):
    """
    A group containing a label, combobox, and optional checkbox
    
    Useful for instancetypes with a preemptible checkbox
    """

    def __init__(self, dialog, **kwargs):
        self.dialog = dialog
        self.has_checkbox = kwargs.get("checkbox")
        self.label = kwargs.get("label", "")
        self.check_label = kwargs.get("check_label", "")

        self.checkbox_id = None
        self.combo_box_id = None
        self.items = []

        self._build()

    def _build(self):
        cols = 3 if self.has_checkbox else 2
        with wutil.grid_group(self.dialog, cols=cols):
            self.combo_box_id = self.dialog.register()
            wutil.dotted_label(self.dialog, self.label)
            self.dialog.AddComboBox(self.combo_box_id, c4d.BFH_SCALEFIT)
            if self.has_checkbox:
                self.checkbox_id = self.dialog.register()
                self.dialog.AddCheckbox(
                    self.checkbox_id, c4d.BFH_RIGHT, 0, 0, self.check_label)

    def clear(self):
        self.dialog.FreeChildren(self.combo_box_id)

    def set_items(self, options, default_value=None):
        """
        Set the combobox to contain the given list of items.

        Strategy to set the selected item:
        1. Existing value if it exists in the list of new items.
        2. Default value if given
        3. First item in the list.

        If the calling function wants to set a different item, it must do so
        explicitly afterwards.
        """
        value = self.get_selected_value()
        if value and value.startswith("--"):
            value = default_value or None
        
        self.clear()
        self.items = []

        for option in options:
            item_id = self.dialog.register()
            self.dialog.AddChild(self.combo_box_id, item_id, option)
            self.items.append((item_id, option))
        
        if not (value and self.set_by_value(value)) :
            self.set_by_index(0)

    def set_by_id(self, item_id):
        self.dialog.SetInt32(self.combo_box_id, item_id)

    def set_by_index(self, index):
        if index < 0:
            index = len(self.items)+index
        self.set_by_id(self.items[index][0])

    def set_by_value(self, value):
        item = next((x for x in self.items if x[1] == value), None)
        if not item:
            return False
        self.set_by_id(item[0])
        return True

    def get_selected_value(self):
        selected_id = self.dialog.GetInt32(self.combo_box_id)
        item = next((x for x in self.items if x[0] == selected_id), None)
        if item:
            return item[1]

    def set_checkbox_value(self, value):
        self.dialog.SetBool(self.checkbox_id, value)

    def get_checkbox_value(self):
        return self.dialog.GetBool(self.checkbox_id)





import c4d
 
from cioc4d.widgets import utils as wutil

class TextFieldGrp(object):
    """A group containing a label, textfield, and optional checkbox.
    
    TODO: Consolidate with HidableTextField.
    """
    def __init__(self, dialog, **kwargs):
        self.dialog = dialog
        self.has_checkbox = kwargs.get("checkbox")
        self.label = kwargs.get("label", "")
        self.check_label = kwargs.get("check_label", "")
        self.helptext = kwargs.get("placeholder")
        self.text_field_id = None
        self.checkbox_id = None

        self._build()

    def _build(self):
        cols = 3 if self.has_checkbox else 2
        with wutil.grid_group(self.dialog, cols=cols):
            wutil.dotted_label(self.dialog, self.label)

            self.text_field_id = self.dialog.register()
            self.dialog.AddEditText(
                self.text_field_id, c4d.BFH_SCALEFIT | c4d.BFV_TOP, editflags=c4d.EDITTEXT_HELPTEXT)

            if self.helptext:
                self.dialog.SetString(
                    self.text_field_id, self.helptext, flags=c4d.EDITTEXT_HELPTEXT)

            if self.has_checkbox:
                self.checkbox_id = self.dialog.register()
                self.dialog.AddCheckbox(
                    self.checkbox_id, c4d.BFH_RIGHT, 0, 0, self.check_label)

    def set_value(self, value):
        self.dialog.SetString(self.text_field_id, value)

    def get_value(self):
        return self.dialog.GetString(self.text_field_id).strip()

    def set_checkbox_value(self, value):
        self.dialog.SetBool(self.checkbox_id, value)

    def get_checkbox_value(self):
        return self.dialog.GetBool(self.checkbox_id)



# -*- coding: utf-8 -*-
import os

import c4d
from cioc4d.sections.collapsible_section import CollapsibleSection
from cioc4d.widgets.combo_box_grp import ComboBoxGrp
from cioc4d.widgets.hidable_text_field_grp import HidableTextFieldGrp
from cioc4d.widgets.text_field_grp import TextFieldGrp
from ciocore import data as coredata
from ciocore.expander import Expander
from ciocore.gpath import Path
from ciocore.gpath_list import PathList

TAKE_OPTIONS = ["Current", "Marked", "Main"]


class GeneralSection(CollapsibleSection):

    ORDER = 10

    def __init__(self, dialog):
        self.takes_widget = None
        self.title_widget = None
        self.projects_widget = None
        self.destination_widget = None
        self.instance_types_widget = None

        super(GeneralSection, self).__init__(dialog, "General", collapse=False)

    def build(self):

        self.takes_widget = ComboBoxGrp(self.dialog, label="Takes")

        self.title_widget = TextFieldGrp(
            self.dialog, label="Job Title", placeholder="Title in Conductor dashboard")

        self.projects_widget = ComboBoxGrp(
            self.dialog, label="Conductor Project")

        self.destination_widget = HidableTextFieldGrp(
            self.dialog, "Override Destination", placeholder="Path where renders are saved to")

        self.dialog.AddSeparatorH(inith=0)

        self.instance_types_widget = ComboBoxGrp(
            self.dialog, label="Instance Type", checkbox=True,
            check_label="Preemptible")

        self.dialog.Enable(self.takes_widget.combo_box_id, False)

    def populate_from_store(self):

        self.update_combo_items()

        # store is guaranteed to be valid
        store = self.dialog.store

        self.instance_types_widget.set_checkbox_value(store.preemptible())
        self.takes_widget.set_items(TAKE_OPTIONS)
        self.takes_widget.set_by_value(store.takes())
        self.title_widget.set_value(store.title())
        self.destination_widget.set_value(store.destination())
        self.destination_widget.set_visible(store.override_destination())

    def save_to_store(self):
        store = self.dialog.store
        store.set_preemptible(self.instance_types_widget.get_checkbox_value())
        store.set_takes(self.takes_widget.get_selected_value())
        store.set_title(self.title_widget.get_value())
        store.set_destination(self.destination_widget.get_value())
        store.set_override_destination(self.destination_widget.get_visible())
        store.set_instance_type(
            self.instance_types_widget.get_selected_value())
        store.commit()

    def on_plugin_message(self, widget_id, msg):
        if widget_id == self.dialog.foot_grp.connect_button_id:
            self.update_combo_items()
        elif widget_id == self.destination_widget.checkbox_id:
            self.destination_widget.set_visible()
        elif widget_id == self.dialog.section("TaskSection").widget.checkbox_id:
            if self.dialog.section("TaskSection").widget.get_visible():
                self.destination_widget.set_visible(True)

        if widget_id in self._store_affectors:
            self.save_to_store()

    def update_combo_items(self):
        if coredata.valid():
            store = self.dialog.store
            projects = coredata.data()["projects"]
            inst_types = coredata.data()["instance_types"]
            self.projects_widget.set_items(
                projects, default_value=store.project())
            self.instance_types_widget.set_items(
                [val["description"] for val in inst_types], default_value=store.instance_type())
        else:
            self.projects_widget.set_items(["-- Not Connected --"])
            self.instance_types_widget.set_items(["-- Not Connected --"])

    def get_selected_instance_type(self):
        if not coredata.valid():
            return
        inst_types = coredata.data()["instance_types"]
        description = self.instance_types_widget.get_selected_value()
        return next((it for it in inst_types if it["description"] == description), None)

    def resolve(self, expander, **kwargs):
        instance_type = self.get_selected_instance_type()

        dest_path = Path(self.resolve_destination_path()).posix_path()

        return {
            "job_title": expander.evaluate(self.title_widget.get_value()),
            "output_path": dest_path,
            "project": self.projects_widget.get_selected_value(),
            "instance_type": instance_type["name"] if instance_type else "INVALID",
            "preemptible": self.instance_types_widget.get_checkbox_value()
        }


    
    def set_store_affectors(self):
        """
        collect widget IDs that trigger store update.

        These are usually the same affectors that trigger a preview. However
        this is a special case because we also need to update the store with the
        destination dir state when the user flips the task visibility switch,
        and since the task visibility switch is not in the preview_affectors
        list for this section, we ust append it.

        You may be wondering, why the task toggle id is not in preview_affectors
        here. It's because the whole preview is updated anyway when the task
        toggle changes so its handled in the TaskSection. Store updates on the
        other hand, only write the one sections properties to the store.
        """
        task_widget_toggle_id = self.dialog.section("TaskSection").widget.checkbox_id

        self._store_affectors = self.get_preview_affectors() + [task_widget_toggle_id]

  
  
    def get_preview_affectors(self):
        """
        
        """
        return [
            self.takes_widget.combo_box_id,
            self.title_widget.text_field_id,
            self.projects_widget.combo_box_id,
            self.destination_widget.text_field_id,
            self.destination_widget.checkbox_id,
            self.instance_types_widget.combo_box_id,
            self.instance_types_widget.checkbox_id
        ]

    def resolve_destination_path(self):
        """
        Resolve the destination path in one of 2 ways.

        1. Get the overridden value
        2. Get the common path of image output dirs.


        """
        context = self.dialog.get_context()
        expander = Expander(safe=True, **context)
        value = None
        if self.destination_widget.get_visible():
            value = self.destination_widget.get_value()
            if value:
                value = expander.evaluate(value)
                if Path(value).relative:
                    value = os.path.join(context["docdir"], value)
        if not value:
            value = self._get_common_destination()
        return value

    def _get_common_destination(self):
        """
        Use common path of outputs for destination directory.
        """
        document = c4d.documents.GetActiveDocument()
        render_data = document.GetActiveRenderData()
        doc_path = document.GetDocumentPath()

        out_paths = list(filter(None, [
            _get_image_save_directory(doc_path, render_data),
            _get_multipass_save_directory(doc_path, render_data)
        ]))
        if not out_paths:
            # fallback
            return os.path.join(doc_path, "renders")

        try:
            return PathList(*out_paths).common_path().posix_path()
        except BaseException:
            return os.path.join(doc_path, "renders")


def _get_image_save_directory(doc_path, render_data):
    save_enabled = render_data[c4d.RDATA_GLOBALSAVE]
    do_image_save = render_data[c4d.RDATA_SAVEIMAGE]
    if save_enabled and do_image_save:
        image_path = render_data[c4d.RDATA_PATH]
        if image_path:
            if Path(image_path).relative:
                return Path(os.path.dirname(os.path.join(doc_path, image_path)))
            return Path(os.path.dirname(image_path))


def _get_multipass_save_directory(doc_path, render_data):
    save_enabled = render_data[c4d.RDATA_GLOBALSAVE]
    do_image_save = render_data[c4d.RDATA_MULTIPASS_SAVEIMAGE]
    if save_enabled and do_image_save:
        image_path = render_data[c4d.RDATA_MULTIPASS_FILENAME]
        if image_path:
            if Path(image_path).relative:
                return Path(os.path.dirname(os.path.join(doc_path, image_path)))
            return Path(os.path.dirname(image_path))



 
import c4d

from cioc4d.widgets.checkbox_grp import CheckboxGrp
from cioc4d.sections.collapsible_section import CollapsibleSection


class UploadOptionsSection(CollapsibleSection):

    ORDER = 80

    def __init__(self, dialog):
        self.use_daemon_widget = None
        self.upload_only_widget = None

        super(UploadOptionsSection, self).__init__(
            dialog, "Upload Options", collapse=True)

    def build(self):
        self.use_daemon_widget = CheckboxGrp(self.dialog, "Use Upload Daemon")
        self.upload_only_widget = CheckboxGrp(self.dialog, "Upload Only")

    def populate_from_store(self):
        store = self.dialog.store
        self.use_daemon_widget.set_value(store.use_upload_daemon())
        self.upload_only_widget.set_value(store.upload_only())

    def save_to_store(self):
        store = self.dialog.store
        store.set_upload_only(self.upload_only_widget.get_value() )
        store.set_use_upload_daemon(self.use_daemon_widget.get_value() )
        store.commit()


    def on_plugin_message(self, widget_id, msg):
        if widget_id in self._store_affectors:
            self.save_to_store()
            
    def resolve(self, _, **kwargs):
        return {
            "local_upload": not self.use_daemon_widget.get_value(),
            "upload_only":  self.upload_only_widget.get_value()
        }
 
    def get_preview_affectors(self):
        return [
            self.use_daemon_widget.checkbox_id,
            self.upload_only_widget.checkbox_id
        ]
 



import re

import c4d
from cioc4d.sections.collapsible_section import CollapsibleSection
from cioc4d.widgets.hidable_text_field_grp import HidableTextFieldGrp
from cioc4d.widgets.int_field_grp import IntFieldGrp
from ciocore.sequence import Sequence
from ciocore.expander import Expander
from cioc4d.conductor_store import DEFAULT_TASK_TEMPLATE

class FramesSection(CollapsibleSection):
    ORDER = 30

    def __init__(self, dialog):

        self.chunk_size_widget = None
        self.custom_range_widget = None
        self.scout_frames_widget = None

        super(FramesSection, self).__init__(dialog, "Frames", collapse=False)

    def build(self):

        self.chunk_size_widget = IntFieldGrp(self.dialog, "Chunk Size")

        self.custom_range_widget = HidableTextFieldGrp(
            self.dialog, "Use Custom Range",
            placeholder="e.g. 1-50x2")

        self.scout_frames_widget = HidableTextFieldGrp(
            self.dialog, "Use Scout Frames",
            placeholder="e.g. auto:3")

    def populate_from_store(self):
        store = self.dialog.store
        self.chunk_size_widget.set_value(store.chunk_size())
        self.custom_range_widget.set_value(store.custom_range())
        self.custom_range_widget.set_visible(store.use_custom_range())
        self.scout_frames_widget.set_value(store.scout_frames())
        self.scout_frames_widget.set_visible(store.use_scout_frames())

    def save_to_store(self):
        store = self.dialog.store
        store.set_chunk_size(self.chunk_size_widget.get_value())
        store.set_custom_range(self.custom_range_widget.get_value())
        store.set_use_custom_range(self.custom_range_widget.get_visible() )
        store.set_scout_frames(self.scout_frames_widget.get_value())
        store.set_use_scout_frames(self.scout_frames_widget.get_visible() )
        store.commit()

    def on_plugin_message(self, widget_id, msg):

        if widget_id == self.chunk_size_widget.int_field_id:
            self.sanitize_chunk_size()
        elif widget_id == self.custom_range_widget.checkbox_id:
            self.custom_range_widget.set_visible()

        elif widget_id == self.scout_frames_widget.checkbox_id:
            self.scout_frames_widget.set_visible()

        if widget_id in self._store_affectors:
            self.save_to_store()

    # TODO: Move some of this logic and the equivalent in ciomaya over to ciocore

    def sanitize_chunk_size(self):
        if self.chunk_size_widget.get_value() < 1:
            self.chunk_size_widget.set_value(1)

    def get_sequence(self):
        """Get a sequence object that represents the c4d frame range.

        If overridden with a custon range, get that instead.
        """
        chunk_size = self.chunk_size_widget.get_value()
        use_custom_range = self.custom_range_widget.get_visible()
        if use_custom_range:
            custom_range = self.custom_range_widget.get_value()
            return Sequence.create(custom_range, chunk_size=chunk_size, chunk_strategy="progressions")

        document = c4d.documents.GetActiveDocument()
        render_data = document.GetActiveRenderData()          # Get the current renderdata
        fps = document.GetFps()
        start_frame = render_data[c4d.RDATA_FRAMEFROM].GetFrame(fps)
        end_frame = render_data[c4d.RDATA_FRAMETO].GetFrame(fps)
        frame_step = render_data[c4d.RDATA_FRAMESTEP]

        return Sequence.create(start_frame, end_frame, frame_step, chunk_size=chunk_size, chunk_strategy="progressions")

    def get_scout_sequence(self, main_sequence):
        """Get a sequence object that represents scout frames if enabled."""
        use_scout_frames = self.scout_frames_widget.get_visible()
        if not use_scout_frames:
            return
        scout_frames = self.scout_frames_widget.get_value()

        match = re.compile(r"^auto[, :]+(\d+)$").match(scout_frames)
        if match:
            num_samples = int(match.group(1))
            return main_sequence.subsample(num_samples)

        try:
            return Sequence.create(scout_frames)
        except (ValueError, TypeError):
            return

    def resolve(self, expander, **kwargs ):
        tasks = []
        max_tasks = kwargs.get("max_tasks", -1)
        if self.dialog.section("TaskSection").widget.get_visible():
            template = self.dialog.section("TaskSection").widget.get_value()
        else:
            template = DEFAULT_TASK_TEMPLATE
        truncate = max_tasks > -1
        sequence = self.get_sequence()
        scout_sequence = self.get_scout_sequence(sequence)
        chunks = sequence.chunks()

        # extract context from the expander so we may create a new expander for
        # each chunk.
        context = expander.context

        for i, chunk in enumerate(chunks):
            if truncate and i >= max_tasks:
                break
            task_context = {
                "start": str(chunk.start),
                "end": str(chunk.end),
                "step": str(chunk.step),
                "chunk_length":  str(len(chunk))
            }
            task_context.update(context)
            this_expander = Expander(safe=True, **task_context)

            tasks.append({
                "command": this_expander.evaluate(template),
                "frames": str(chunk)
            })

        return {
            "tasks_data": tasks,
            "scout_frames": ",".join([str(s) for s in scout_sequence or []])}

    def get_preview_affectors(self):
        return [
            self.chunk_size_widget.int_field_id,
            self.custom_range_widget.checkbox_id,
            self.custom_range_widget.text_field_id,
            self.scout_frames_widget.checkbox_id,
            self.scout_frames_widget.text_field_id
        ]
 



import c4d
import os
import json
from contextlib import contextmanager
from ciocore import conductor_submit
from ciocore import CONFIG
import urlparse
import traceback
from ciocore.expander import Expander
from cioc4d import asset_cache
from ciocore.validator import Validator, ValidationError
from cioc4d import validation
 
SAVE_AS_DIALOG = 12218


@contextmanager
def transient_save(filepath, cleanup=True):
    doc = c4d.documents.GetActiveDocument()
    originalname = doc.GetDocumentName()
    originalpath = doc.GetDocumentPath()

    newname = os.path.basename(filepath)
    newpath = os.path.dirname(filepath)

    doc.SetDocumentName(newname)
    doc.SetDocumentPath(newpath)
    
    c4d.documents.SaveDocument(
        doc,
        filepath,
        saveflags=c4d.SAVEDOCUMENTFLAGS_AUTOSAVE,
        format=c4d.FORMAT_C4DEXPORT)
    doc.SetDocumentName(newname)
    doc.SetDocumentPath(newpath)

    yield
    doc.SetDocumentName(originalname)
    doc.SetDocumentPath(originalpath)
    
    if cleanup:
        try:
            os.remove(filepath)
        except OSError:
            c4d.WriteConsole("Couldn't cleanup file: {}".format(filepath))

def valid(dialog):
    try:
        validation.run(dialog)
    except ValidationError as ex:
        c4d.WriteConsole(str(ex))
        return False
    return True



def submit(dialog):
    asset_cache.clear()

    filepath, cleanup = _resolve_autosave_template(dialog)
    if filepath:
        with transient_save(filepath, cleanup=cleanup):
            # if valid(dialog):
            handle_submissions(dialog)
        return

    if needs_save():
        c4d.CallCommand(SAVE_AS_DIALOG)
    if needs_save():
        return
    # if valid(dialog):
    handle_submissions(dialog)


def needs_save():
    doc = c4d.documents.GetActiveDocument()
    return doc.GetChanged() or '' == doc.GetDocumentPath()


def _resolve_autosave_template(dialog):
    context = dialog.get_context()
    autosave_widget = dialog.section("AutosaveSection").widget
    do_autosave = autosave_widget.get_visible()
    template = autosave_widget.get_value()
    cleanup = autosave_widget.get_extra_check_value() and not dialog.section(
        "UploadOptionsSection").use_daemon_widget.get_value()

    if not context.get("docdir") and do_autosave:
        return (None, None)

    resolved_name = Expander(safe=True, **context).evaluate(template)

    return (resolved_name, cleanup)


def handle_submissions(dialog):
    submission = dialog.calculate_submission(max_tasks=-1, with_assets=True)
    if valid(dialog):
        response = do_submission(dialog, submission)
        show_submission_response(response)


def do_submission(dialog, submission):
 
    show_tracebacks = dialog.section("DiagnosticsSection").widget.get_value()
 
    try:
        remote_job = conductor_submit.Submit(submission)
        response, response_code = remote_job.main()
        return {"code": response_code, "response": response}
    except BaseException as ex:
        if show_tracebacks:
            msg = traceback.format_exc()
        else:
            msg = ex.message
        c4d.WriteConsole(msg)
        return{"code": "undefined", "response": msg}


def show_submission_response(response):
    if response.get("code") <= 201:
        # success
        success_uri = response["response"]["uri"].replace("jobs", "job")
        job_url = urlparse.urljoin(CONFIG["auth_url"], success_uri)

        c4d.WriteConsole("Use this URL to monitor your Conductor job {}".format(job_url))
        c4d.gui.MessageDialog( "Success: {}".format(job_url))
        return

    c4d.gui.MessageDialog( "Failure: {}".format(str(response["response"])))




"""
Collapsible Group is a base class for each stacked section of UI 
"""
# import os

import c4d
import abc

TOGGLE_W = 100
TOGGLE_X = 11
TOGGLE_Y = 7

TOGGLE_DIST = 5
DEFAULT_TEXT = 191/255.0
COLLAPSED_COLOR = c4d.Vector(0.5, DEFAULT_TEXT, 1)
EXPANDED_COLOR = c4d.Vector(1, DEFAULT_TEXT, DEFAULT_TEXT)
DEFAULT_COLOR = c4d.Vector(DEFAULT_TEXT, DEFAULT_TEXT, DEFAULT_TEXT)


class CollapsibleSection(object):
    """
    Provide a titled colapsible section of UI.

    C4Ds own fold flag has been broken for years. This class abstracts away the
    mechanics required to expand and collapse the contained UI.

    By using this as a section base class we get other benefits, like
    consistency of look and feel, event handling, and introspection. 
    """
    __metaclass__ = abc.ABCMeta

    @abc.abstractproperty
    def ORDER(self):
        """
        Subclasses must provide their place in the stacking order. 

        Values can be sparse: e.g. 10, 20, 30.
        """
        pass

    def __init__(self, dialog, title, collapse=False, override_color=None):
        """
        Initialize the section. 

        Call the derived class build method, which places specific widgets
        inside the inner group.
        """
        self.group_id = None
        self.vis_group_id = None
        self.separator_id = None
        self.collapsed = None
        self.title = title
        self.dialog = dialog
        self._store_affectors = []

        self._collapsed_color=override_color or COLLAPSED_COLOR
        self._expanded_color=override_color or EXPANDED_COLOR

        # Build the UI
        if not self._begin_group():
            return
        self.build()
        self._end_group()

        self.collapse() if collapse else self.expand()

    @abc.abstractmethod
    def build(self):
        """
        Derived class overrides build. 

        Places widgets.
        """
        return


    def populate_from_store(self):
        pass

    def save_to_store(self):
        pass


    def on_plugin_message(self, widget_id, msg):
        """
        Handle events from this plugin.
        """
        pass

    def on_core_message(self, msg_id, msg):
        """
        Handle core events from c4d.
        """
        pass

    def on_message(self, msg_id, msg):
        """
        Handle other messages from c4d.

        We detect mouse events in order to collapse/expand. If a subclass
        implements on_message(), it should be sure to call this base class
        method too. 
        """
        if msg_id == c4d.BFM_INPUT:
            self.on_mouse_click(msg)

    def _begin_group(self):
        """
        Set up the collapsible group structure.

        The outer group displays the clickable label and border. The inner group
        is hidable and contains all subclass widgets.

        TODO: convert to contextmanager
        """
        self.group_id = self.dialog.register()
        title = self.get_titled_toggle()

        # OUTER
        grp = self.dialog.GroupBegin(
            id=self.group_id,
            flags=c4d.BFH_SCALEFIT,
            title=title,
            rows=1,
            cols=1,
            groupflags=0)
        if not grp:
            return

        self.dialog.GroupBorderSpace(4, 4, 4, 4)
        self.dialog.GroupBorder(c4d.BORDER_WITH_TITLE | c4d.BORDER_ROUND)

        # must add an element here (with ID) to prevent a bug where an empty
        # rounded border grp doesn't know how to display properly
        self.separator_id = self.dialog.register()
        self.dialog.AddStaticText(self.separator_id, c4d.BFH_SCALEFIT, inith=4)

        # INNER
        self.vis_group_id = self.dialog.register()
        grp = self.dialog.GroupBegin(
            id=self.vis_group_id,
            cols=1,
            flags=c4d.BFH_SCALEFIT,
            groupflags=0)
        if not grp:
            return

        self.dialog.SetDefaultColor(
            self.vis_group_id, c4d.COLOR_TEXT,   DEFAULT_COLOR)

        return True

    def _end_group(self):
        """End the above groups."""
        self.dialog.GroupEnd()
        self.dialog.GroupEnd()

    def get_titled_toggle(self):
        """Construct label."""
        toggler = "+" if self.collapsed else "-"

        return "{} {}".format(toggler, self.title)

    def on_mouse_click(self, msg):
        """
        Determine if mouse click was on the label.

        If so, toggle collapse/expand
        """
        mousex = msg.GetInt32(c4d.BFM_INPUT_X)
        mousey = msg.GetInt32(c4d.BFM_INPUT_Y)
        group_dim = self.dialog.GetItemDim(self.group_id)
        y = (group_dim["y"]+TOGGLE_Y) - mousey
        if abs(y) > TOGGLE_DIST:
            return
        x = (group_dim["x"]+TOGGLE_X) - mousex
        if x > TOGGLE_DIST:
            return
        x = mousex - (group_dim["x"]+TOGGLE_X + TOGGLE_W)
        if x > TOGGLE_DIST:
            return
        self.toggle()

    def collapse(self):
        """
        Set section to collapsed.

        Hide inner group, but show the spacer so that the outer group has
        something to keep it from bugging out. 

        Change the color to night.
        """
        self.dialog.HideElement(self.vis_group_id, True)
        self.dialog.HideElement(self.separator_id, False)
        self.collapsed = True
        self.dialog.SetString(self.group_id, self.get_titled_toggle())
        self.dialog.SetDefaultColor(
            self.group_id, c4d.COLOR_TEXT,   self._collapsed_color)
        self.dialog.LayoutChanged(self.group_id)
        self.on_collapse()
        return self.collapsed

    def expand(self):
        """
        Set section to expanded.

        Show inner group, but hide the spacer so that it doesn't add extra
        unwanted space while group is expanded. 

        Change the color to day.
        """
        self.dialog.HideElement(self.vis_group_id, False)
        self.dialog.HideElement(self.separator_id, True)

        self.collapsed = False
        self.dialog.SetString(self.group_id, self.get_titled_toggle())
        self.dialog.SetDefaultColor(
            self.group_id, c4d.COLOR_TEXT,   self._expanded_color)
        self.dialog.LayoutChanged(self.group_id)
        self.on_expand()
        return self.collapsed

    def toggle(self):
        """Flip expand/collapsed state """
        return self.expand() if self.collapsed else self.collapse()

    def on_collapse(self):
        pass

    def on_expand(self):
        pass

    def resolve(self, _=None, **kwargs):
        return {}

    def get_preview_affectors(self):
        """
        Return widget IDs that trigger preview resolution.

         
        """
        return []


    def set_store_affectors(self):
        """
        Save widget IDs that trigger store update.

        These will most likely be the same affectors that trigger a preview. If
        they are not, then override this method.
        """
        self._store_affectors = self.get_preview_affectors()
  
  




# encoding: utf-8
import c4d
from cioc4d.widgets import utils as wutil


class KeyValueHeader(object):
    """A widget containing a header to sit above a KeyValueWidget."""

    def __init__(self, dialog, checkbox_label=None):

        self.checkbox_label = checkbox_label
        self.dialog = dialog
        self.group_id = None

        self.add_button_id = None
        self.name_text_id = None
        self.value_text_id = None
        self.extra_text_id = None

        self._build()

    def _build(self):

        with wutil.grid_group(self.dialog, cols=4) as group_id:
            self.group_id = group_id

            self.add_button_id = self.dialog.register()
            self.dialog.AddButton(
                self.add_button_id, c4d.BFH_FIT, initw=20,  name="Add")

            self.name_text_id = self.dialog.register()
            self.dialog.AddStaticText(
                id=self.name_text_id, initw=215, name="Name",
                borderstyle=c4d.BORDER_TEXT_DOTTED, flags=c4d.BFH_FIT)

            self.value_text_id = self.dialog.register()
            self.dialog.AddStaticText(
                id=self.name_text_id, name="Value",
                borderstyle=c4d.BORDER_TEXT_DOTTED, flags=c4d.BFH_SCALEFIT)

            if self.checkbox_label:
                self.extra_text_id = self.dialog.register()
                self.dialog.AddStaticText(
                    id=self.name_text_id, name=self.checkbox_label,
                    borderstyle=c4d.BORDER_TEXT_DOTTED, flags=c4d.BFH_FIT | c4d.BFH_RIGHT, initw=40)


class KeyValueWidget(object):
    """A widget containing key/value, textfields, and optional checkbox."""

    def __init__(self, dialog, *values):

        self.has_checkbox = len(values) == 3
        self.dialog = dialog
        self.group_id = None
        self.name_field_id = None
        self.value_field_id = None
        self.extra_checkbox_id = None
        self.delete_btn_id = None
        self._build()
        self.set_values(*values)

    def _build(self):

        with wutil.grid_group(self.dialog, cols=4) as group_id:
            self.group_id = group_id

            self.delete_btn_id = self.dialog.register()
            self.dialog.AddButton(self.delete_btn_id,
                                  c4d.BFH_FIT, initw=20, name='X')

            self.name_field_id = self.dialog.register()
            self.dialog.AddEditText(
                self.name_field_id, c4d.BFH_FIT | c4d.BFV_TOP, initw=200)

            self.value_field_id = self.dialog.register()
            self.dialog.AddEditText(
                self.value_field_id, c4d.BFH_SCALEFIT | c4d.BFV_TOP)
            if self.has_checkbox:
                self.extra_checkbox_id = self.dialog.register()
                self.dialog.AddCheckbox(
                    self.extra_checkbox_id,  c4d.BFH_FIT | c4d.BFH_RIGHT, initw=35, inith=0, name="")

    def set_values(self, *values):
        self.dialog.SetString(self.name_field_id, values[0])
        self.dialog.SetString(self.value_field_id, values[1])
        if self.has_checkbox:
            self.dialog.SetBool(self.extra_checkbox_id, values[2])

    def get_values(self):
        if self.has_checkbox:
            return (
                self.dialog.GetString(self.name_field_id).strip(),
                self.dialog.GetString(self.value_field_id).strip(),
                self.dialog.GetBool(self.extra_checkbox_id)
            )
        return (
            self.dialog.GetString(self.name_field_id).strip(),
            self.dialog.GetString(self.value_field_id).strip()
        )



"""
Contains definitions of common constants.


This file mustr be kept at the same level as the plugin "pyp" file so we can extract the version.
"""
import os
import re
import c4d

LABEL_WIDTH = 180
LABEL_HEIGHT = 14
INT_FIELD_WIDTH = 100
PLUGIN_ID = 1055243 # registered for Conductor at Maxon


C4D_VERSION = int(c4d.GetC4DVersion() / 1000)
 

PLUGIN_DIR = os.path.dirname(__file__)
VERSION = u"dev.999"
try:
    with open(os.path.join(PLUGIN_DIR, "VERSION")) as version_file:
        VERSION = version_file.read().strip()
except BaseException:
    pass






from contextlib import contextmanager

import c4d


@contextmanager
def footer_group(dialog, **kwargs):
    grp_id = dialog.register()
    grp = dialog.GroupBegin(
        id=grp_id,
        flags=c4d.BFH_CENTER | c4d.BFV_BOTTOM,
        title="",
        rows=1,
        cols=kwargs.get("cols", 2),
        groupflags=0)
    if grp:
        dialog.GroupBorderSpace(5, 5, 5, 5)
        yield grp_id
    dialog.GroupEnd()


class FootGroup(object):

    def __init__(self, dialog):
        self.group_id = None
        self.dialog = dialog
        self.connect_button_id = None
        self.reset_button_id = None
        self.validate_button_id = None
        self.submit_button_id = None
        self._build()

    def _build(self):
        with footer_group(self.dialog, cols=4) as group_id:
            self.group_id = group_id
            self.connect_button_id = self.dialog.register()
            self.submit_button_id = self.dialog.register()
            self.validate_button_id = self.dialog.register()
            self.reset_button_id = self.dialog.register()

            self.dialog.AddButton(self.connect_button_id,
                                  c4d.BFV_CENTER,  name="Connect")
            self.dialog.AddButton(self.reset_button_id,
                                  c4d.BFV_CENTER,  name="Reset UI")
            self.dialog.AddButton(self.validate_button_id,
                                  c4d.BFV_CENTER,  name="Validate")
            self.dialog.AddButton(self.submit_button_id,
                                  c4d.BFV_CENTER,   name="Submit")
            self.enable_submit(False)
    def enable_submit(self, enable):
        self.dialog.Enable(self.submit_button_id, enable)
 


import abc
import importlib
import inspect
import os
import re

import c4d
from cioc4d import asset_cache
# from ciocore.expander import Expander
from ciocore.gpath import Path
from ciocore.gpath_list import GLOBBABLE_REGEX, PathList
from ciocore.validator import ValidationError, Validator


class ValidateUploadDaemon(Validator):
    def run(self, _):
        dialog = self._submitter
        use_daemon = dialog.section(
            "UploadOptionsSection").use_daemon_widget.get_value()
        if not use_daemon:
            return

        msg = "This submission expects an uploader daemon to be running.\n After you press submit you can open a shell and type: 'conductor uploader'"

        location = (dialog.section(
            "LocationSection").widget.get_value() or "").strip()
        if location:
            msg = "This submission expects an uploader daemon to be running and set to a specific location tag.\nAfter you press OK you can open a shell and type: 'conductor uploader --location {}'".format(
                location)

        self.add_notice(msg)
        # By also writing the message to the console, the user can copy paste
        # `conductor uploader --location blah`.
        c4d.WriteConsole(msg)


class ValidateTaskCount(Validator):
    def run(self, _):
        dialog = self._submitter
        count = dialog.section("InfoSection").frame_count
        if count > 1000:
            self.add_notice(
                "This submission contains over 1000 tasks ({}). Are you sure this is correct?".format(count))


class ValidateDestinationDirectoryClash(Validator):
    def run(self, _):
        dialog = self._submitter

        dest = dialog.section("GeneralSection").resolve_destination_path()

        # Validate path containment without drive letters
        dest_path = Path(dest).posix_path(with_drive=False)
 
        for gpath in asset_cache.data(self._submitter):
            asset_path = gpath.posix_path(with_drive=False)
            if asset_path.startswith(dest_path):
                c4d.WriteConsole("Some of your upload assets exist in the specified output destination directory\n. {} contains {}".format(
                    dest_path, asset_path))
                self.add_error(
                    "The destination directory for rendered output contains assets that are in the upload list. This can cause your render to fail. See the script editor for details.")
                break
            if dest_path.startswith(asset_path):
                c4d.WriteConsole("You are trying to upload a directory that contains your destination directory.\n. {} contains {}".format(
                    asset_path, dest_path))
                self.add_error(
                    "One of your assets is a directory that contains the specified output destination directory. This will cause your render to fail. See the script editor for details.")
                break


class ValidateRenderToDestinationDirectory(Validator):
    def run(self, _):
        # TODO, break this up into its component parts

        dialog = self._submitter
        # context = dialog.get_context()
        # expander = Expander(safe=True, **context)


        dest = dialog.section("GeneralSection").resolve_destination_path()

        dest_path = Path(dest).posix_path(with_drive=False)

        task_template = dialog.section("TaskSection").widget.get_value()

        document = c4d.documents.GetActiveDocument()

        render_data = document.GetActiveRenderData()
        if not render_data[c4d.RDATA_GLOBALSAVE]:
            self.add_warning(
                "Saving images is not enabled in Render settings. Your render will most likely fail.")
            return

        do_image_save = render_data[c4d.RDATA_SAVEIMAGE]
        do_multipass_save = render_data[c4d.RDATA_MULTIPASS_SAVEIMAGE] and render_data[c4d.RDATA_MULTIPASS_ENABLE]

        self._validate_image(
            "image",
            "-oimage",
            task_template,
            do_image_save,
            c4d.RDATA_PATH,
            dest_path,
            document,
            render_data
        )

        self._validate_image(
            "multi-image",
            "-omultipass",
            task_template,
            do_multipass_save,
            c4d.RDATA_MULTIPASS_FILENAME,
            dest_path,
            document,
            render_data
        )

    def _validate_image(
            self,
            label,
            override_flag,
            task_template,
            do_save,
            c4d_filename_id,
            dest_path,
            document,
            render_data):


        common_help =   """If you continue, your submission will probably fail. 
Images should be written somewhere below the Destination Directory. 
You should change either the path in Render Settings, or the Conductor destination directory."""


        if override_flag in task_template:
            self.add_notice(
                """Your task template overrides the Render Settings image path with the {} flag. 
This is fine, but please ensure it points to a writable directory. 
It should be somewhere inside the Destination Directory. Check the Preview pane.""".format(override_flag))
            return

        image_path = render_data[c4d_filename_id]

        if not (image_path and do_save):
            self.add_notice(
                "This file does not specify an output {}. ".format(label))
            return

        try:
            if Path(image_path).relative:
                image_path = os.path.join(
                    document.GetDocumentPath(), image_path)

            image_path = Path(image_path).posix_path(with_drive=False)

            if not image_path.startswith(dest_path):
                self.add_warning(
                    """The {} path you've set in Render Settings does not point to a writable location.
{}""".format(label, common_help))
            return
        except ValueError:
            self.add_warning(
                """The {} path set in Render Settings is not a valid path.
{}""".format(label, common_help))

class ValidateMissingExtraAssets(Validator):

    def run(self, _):

        missing = []
        for gpath in self._submitter.section("AssetsSection").pathlist:
            pp = gpath.posix_path()
            if not os.path.exists(pp):
                missing.append(pp)

        if missing:
            self.add_warning(
                "Some of the assets specified in the Extra Assets section do not exist on disk. See the console for details. You can continue if you don't need them.")

            c4d.WriteConsole("----- Conductor Asset Validation -------\n")
            for asset in missing:
                c4d.WriteConsole("Missing: {}\n".format(asset))


class ValidateDontSaveVideoPosts(Validator):

    GI_AUTOSAVE_ID = 3804
    AO_AUTOSAVE_ID = 2204
    AO_USE_CACHE_ID = 2000

    def run(self, _):
        document = c4d.documents.GetActiveDocument()
        render_data = document.GetActiveRenderData()
        vp = render_data.GetFirstVideoPost()
        while (vp):
            if vp.CheckType(c4d.VPglobalillumination):
                self._validate_video_post(
                    vp, "Global Illumination", self.GI_AUTOSAVE_ID)
            elif vp.CheckType(c4d.VPambientocclusion):
                self._validate_video_post(
                    vp, "Ambient occlusion", self.AO_AUTOSAVE_ID, self.AO_USE_CACHE_ID)
            vp = vp.GetNext()

    def _validate_video_post(self, vp, label, *ids):
        if vp.GetBit(c4d.BIT_VPDISABLED):
            return
        container = vp.GetDataInstance()
        for element_id in ids:
            if not container[element_id]:
                return

        self.add_warning(
            "{} Auto Save is set to ON. You should turn it off for the render, otherwise it may try to write files in a read-only directory and cause the render to fail".format(label))


# Implement more validators here
####################################
####################################


def run(dialog, submitting=True):

    errors, warnings, notices = _run_validators(dialog)

    if errors:
        msg = ""
        msg += "Some errors would cause the submission to fail:\n\n" + \
            "\n".join(errors)+"\n"
        c4d.gui.MessageDialog(msg,  type=c4d.GEMB_OK)
        raise ValidationError(msg)
    if notices or warnings:
        if submitting:
            msg = "Would you like to continue this submission?\n\n"
            dialog_type = c4d.GEMB_OKCANCEL
        else:
            msg = "Validate only.\n\n"
            dialog_type = c4d.GEMB_OK

        if warnings:
            msg += "Please check the warnings below:\n\n" + \
                "\n\n".join(["[WARN]:{}".format(w) for w in warnings]) + "\n\n"
        if notices:
            msg += "Please check the notices below:\n\n" + \
                "\n\n".join(["[INFO]:{}".format(n) for n in notices]) + "\n\n"

        result = c4d.gui.MessageDialog(msg, type=dialog_type)
        if result != c4d.GEMB_R_OK:
            c4d.WriteConsole("Submission cancelled by user.\n")
            raise ValidationError(msg)
    # Either there were no  messages, or the user clicked Continue (OK)


def _run_validators(dialog):
    # Eventually we will handle takes here
    takename = "Main"
    validators = [plugin(dialog) for plugin in Validator.plugins()]
    for validator in validators:
        validator.run(takename)

    errors = list(set.union(*[validator.errors for validator in validators]))
    warnings = list(
        set.union(*[validator.warnings for validator in validators]))
    notices = list(set.union(*[validator.notices for validator in validators]))
    return errors, warnings, notices



#!/usr/bin/env python
# -*- coding: utf-8 -*-



import os
import setuptools
from PIL import Image

from setuptools.command.build_py import build_py


NAME = "cioc4d"
DESCRIPTION = "C4D plugin for Conductor Cloud Rendering Platform."
URL = "https://github.com/AtomicConductor/conductor-c4d"
EMAIL = "info@conductortech.com"
AUTHOR = "conductor"
REQUIRES_PYTHON = "~=2.7"
REQUIRED = ["ciocore>=0.2.7,<1.0.0"]
HERE = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(HERE, 'VERSION')) as version_file:
    VERSION = version_file.read().strip()

class BuildCommand(build_py):
    def run(self):
        build_py.run(self)

        if not self.dry_run:
            target_dir = os.path.join(self.build_lib, NAME)
            self.write_version_info(target_dir)
            self.convert_icons(target_dir)

    def write_version_info(self, target_dir):
        with open(os.path.join(target_dir, "VERSION"), "w") as f:
            f.write(VERSION)

    def convert_icons(self, target_dir):
        """Convert icons using PIL."""
        out_icon_dir = os.path.join(target_dir, "res")
        icons_dir = os.path.join("images", "icons")
        icon_files = os.listdir(icons_dir)
        for icon_file in icon_files:
            base, ext = os.path.splitext(icon_file)
            icon_path = os.path.join(icons_dir, icon_file)
            size = 30
            icon = Image.open(icon_path)
            icon.thumbnail((size, size), Image.BICUBIC)
            out_icon_file = os.path.join(
                out_icon_dir, "{}_{}x{}{}".format(base, size, size, ext))
            icon.save(out_icon_file)
 
setuptools.setup(
    author=AUTHOR,
    author_email=EMAIL,
    classifiers=[
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python",
        "Topic :: Multimedia :: Graphics :: 3D Rendering",
    ],
    cmdclass={"build_py": BuildCommand},
    description=DESCRIPTION,
    install_requires=REQUIRED,
    long_description=DESCRIPTION,
    long_description_content_type="text/markdown",
    name=NAME,
    package_dir={"": "."},
    packages=setuptools.find_packages(where="."),
    include_package_data=True,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    version=VERSION,
    zip_safe=False,
    package_data={NAME: ["res/*.*", "res/*/*.*", "res/*/*/*.*", "Conductor.pyp", ]}
)





import c4d

from cioc4d.widgets.hidable_text_field_grp import HidableTextFieldGrp
from cioc4d.sections.collapsible_section import CollapsibleSection

class TaskSection(CollapsibleSection):

    ORDER = 50

    def __init__(self, dialog):
        self.widget = None
        super(TaskSection, self).__init__(dialog, "Task Template", collapse=True)

    def build(self):
        self.widget = HidableTextFieldGrp(self.dialog, label="Override Template", placeholder="Task command template")



    def populate_from_store(self):
        store = self.dialog.store
        self.widget.set_value(store.task_template())
        self.widget.set_visible(store.override_task_template())

    def save_to_store(self):
        store = self.dialog.store
        store.set_task_template(self.widget.get_value())
        store.set_override_task_template(self.widget.get_visible())
        store.commit()

         
    def get_preview_affectors(self):
        return [self.widget.text_field_id, self.widget.checkbox_id]

    
    def on_plugin_message(self, widget_id, msg):
    
        if widget_id == self.widget.checkbox_id:
            self.widget.set_visible()

        if widget_id in self._store_affectors:
            self.save_to_store()



from contextlib import contextmanager

import c4d


@contextmanager
def head_group(dialog):
    grp = dialog.GroupBegin(
        id=dialog.register(),
        flags=c4d.BFH_SCALEFIT,
        rows=1,
        cols=2,
        groupflags=0)
    if grp:
        yield
    dialog.GroupEnd()


class HeadGroup(object):

    def __init__(self, dialog):
        self.dialog = dialog
        self._build()

    def _build(self):
        with head_group(self.dialog):
            self.dialog.AddGadget(c4d.DIALOG_PIN, 0)
            self.dialog.AddStaticText(
                id=self.dialog.register(),
                initw=0,
                inith=0,
                borderstyle=0,
                flags=c4d.BFH_SCALEFIT)





import c4d
import cioc4d.const as k
from cioc4d.widgets import utils as wutil


class IntFieldGrp(object):
    """A label and int field."""

    def __init__(self, dialog, label):
        self.dialog = dialog
        self.label = label
        self.int_field_id = None

        self._build()

    def _build(self):

        with wutil.grid_group(self.dialog, cols=2):

            wutil.dotted_label(self.dialog, self.label)

            self.int_field_id = self.dialog.register()
            self.dialog.AddEditNumberArrows(
                self.int_field_id, c4d.BFH_LEFT | c4d.BFV_TOP, initw=k.INT_FIELD_WIDTH)

    def set_value(self, value):
        self.dialog.SetInt32(self.int_field_id, value)

    def get_value(self):
        return self.dialog.GetInt32(self.int_field_id)
